<?php
namespace Kuoqi\Worker;
use Kuoqi\Error;

/**
 * worker的行为初始化信息
 * Class Behavior
 * @package Kuoqi\Worker
 */
class Behavior extends \Kuoqi\KuoqiException {
    /**
     * @var string
     * domain 参数指定哪个协议用在当前套接字上。
     * 可用的地址/协议
     * Domain
     * 描述
     * AF_INET IPv4 网络协议。TCP 和 UDP 都可使用此协议。
     * AF_INET6 IPv6 网络协议。TCP 和 UDP 都可使用此协议。
     * AF_UNIX 本地通讯协议。具有高性能和低成本的 IPC（进程间通讯）。
     */
    protected $node_domain = AF_INET;
    /**
     * @var string
     * type 参数用于选择套接字使用的类型。
     * 可用的套接字类型
     * 类型
     * 描述
     * SOCK_STREAM 提供一个顺序化的、可靠的、全双工的、基于连接的字节流。支持数据传送流量控制机制。TCP 协议即基于这种流式套接字。
     * SOCK_DGRAM 提供数据报文的支持。(无连接，不可靠、固定最大长度).UDP协议即基于这种数据报文套接字。
     * SOCK_SEQPACKET 提供一个顺序化的、可靠的、全双工的、面向连接的、固定最大长度的数据通信；数据端通过接收每一个数据段来读取整个数据包。
     * SOCK_RAW 提供读取原始的网络协议。这种特殊的套接字可用于手工构建任意类型的协议。一般使用这个套接字来实现 ICMP 请求（例如 ping）。
     * SOCK_RDM 提供一个可靠的数据层，但不保证到达顺序。一般的操作系统都未实现此功能。
     */
    protected $node_type = SOCK_STREAM;
    /**
     * @var string
     * protocol
     * protocol 参数，是设置指定 domain 套接字下的具体协议。这个值可以使用 getprotobyname() 函数进行读取。如果所需的协议是 TCP 或 UDP，可以直接使用常量 SOL_TCP 和 SOL_UDP 。
     * 常见协议
     * 名称
     * 描述
     * icmp Internet Control Message Protocol 主要用于网关和主机报告错误的数据通信。例如"ping"命令（在目前大部分的操作系统中）就是使用 ICMP 协议实现的。
     * udp User Datagram Protocol 是一个无连接的、不可靠的、具有固定最大长度的报文协议。由于这些特性，UDP 协议拥有最小的协议开销。
     * tcp Transmission Control Protocol 是一个可靠的、基于连接的、面向数据流的全双工协议。TCP 能够保障所有的数据包是按照其发送顺序而接收的。如果任意数据包在通讯时丢失，TCP 将自动重发数据包直到目标主机应答已接收。因为可靠性和性能的原因，TCP 在数据传输层使用 8bit 字节边界。因此，TCP 应用程序必须允许传送部分报文的可能。
     */
    protected $node_protocol = SOL_TCP;

    public $node = null;//通讯节点s
    protected $server_addr = '';//服务地址
    protected $server_port = '';//服务端口
    /**
     * @var int 请求堆积的最大数量
     * 最多将留下积压的传入连接进行处理。如果连接请求到达时队列已满，则客户端可能会收到一个错误，指示为ECONNREFUSED，或者如果基础协议支持重新传输，则可以忽略该请求，以便重试成功。
     * note : 传递给backlog参数的最大数量高度取决于底层平台。在Linux上，它被截断为SOMAXCONN。在win32上，如果传递了SOMAXCONN，则负责套接字的底层服务提供者将将backlog设置为最大合理值。在这个平台上，没有标准条款来确定实际的积压价值。
     */
    protected $backlog = SOMAXCONN;

    /**
     * 创建通讯节点
     * @return bool
     */
    protected function CreateNodeCommunication(){
        $this->node = socket_create($this->node_domain, $this->node_type, $this->node_protocol);
        if($this->node===false){
            $this->errcode = -1;
            $this->errmsg = sprintf("CreateNodeCommunication() failed with %s",mb_convert_encoding(socket_strerror(socket_last_error()),'UTF-8',"GB2312"));
            return false;
        }else{
            return true;
        }
    }
    /**
     * 给节点绑定一个名字
     * @return bool
     * 绑定 address 到 socket。该操作必须是在使用 socket_connect() 或者 socket_listen() 建立一个连接之前。
     * socket
     * 用 socket_create() 创建的一个有效的套接字资源。
     * address
     * 如果套接字是 AF_INET 族，那么 address 必须是一个四点分法的 IP 地址（例如 127.0.0.1 ）。
     *
     * 如果套接字是 AF_UNIX 族，那么 address 是 Unix 套接字一部分（例如 /tmp/my.sock ）。
     * port （可选）
     * 参数 port 仅仅用于 AF_INET 套接字连接的时候，并且指定连接中需要监听的端口号。
     * 成功时返回 TRUE， 或者在失败时返回 FALSE。
     *
     * 错误代码会传入 socket_last_error() ，如果将此参数传入 socket_strerror() 则可以得到错误的文字说明。
     *
     */
    protected function CreateNodeBind(){
        if(@socket_bind($this->node,$this->server_addr,$this->server_port)){
            return true;
        }else{
            $this->errmsg = sprintf('CreateNodeBind warning:socket_bind failed on %s:%s with %s'."\n",
                $this->server_addr,
                $this->server_port,
                mb_convert_encoding(socket_strerror(socket_last_error()),'UTF-8',"GB2312")
            );
            $this->errcode = -1;
            return false;
        }
    }
    /**
     * 创建节点监听
     * @return bool
     */
    protected function CreateNodeListen(){
        if(socket_listen($this->node,$this->backlog)){
            return true;
        }else{
            $this->errmsg = "socket_listen() failed";
            $this->errcode = -1;
            return false;
        }
    }

    /**
     * 执行监听的内容部分
     */
    public function listen(){
        $this->log('webSocket Listen . you need respire this function to your context');
        sleep(5);
    }
    public function log($msg){
        //如果有GUI则展示
        echo $msg.PHP_EOL;
        //return \Kuoqi::log($msg);
    }
    public function error($msg){
        throw new \Kuoqi\Error( mb_convert_encoding($msg,'UTF-8',"GB2312"));
    }
    public function start(){
        $this->run();
    }
    //执行运行
    public function run(){
        while(true){
            $this->listen();
        }
    }
}